#ifndef _BSP_UTILS_H
#define _BSP_UTILS_H

/*
===========
split
	do the actual poly split into
	front and back pieces
===========
*/
void split(const Poly &poly, const Plane &plane, Poly &front_split, Poly &back_split) {
	register int offsetA, offsetB;
	double dA, dB, scalar;
	Point3D isect;
	// loop through vertexes to find splits
	for(int i=0; i<poly.num_vertexes; i++) {
		offsetA=i;
		offsetB=(i+1) % poly.num_vertexes;
		dA=poly.vertex[offsetA].dot(plane.normal);
		dB=poly.vertex[offsetB].dot(plane.normal);
		if((dA>=plane.dist_origin) == (dB>=plane.dist_origin)) {
			if(dA>=plane.dist_origin) {
				front_split.add(poly.vertex[offsetA]);
				front_split.add(poly.vertex[offsetB]);
			}
			else {
				back_split.add(poly.vertex[offsetA]);
				back_split.add(poly.vertex[offsetB]);
			}
		}
		else {
			scalar = (plane.dist_origin-dA) / (dB-dA);
			isect.setx(poly.vertex[offsetA].getx() + (poly.vertex[offsetB].getx() - poly.vertex[offsetA].getx())*scalar);
			isect.sety(poly.vertex[offsetA].gety() + (poly.vertex[offsetB].gety() - poly.vertex[offsetA].gety())*scalar);
			isect.setz(poly.vertex[offsetA].getz() + (poly.vertex[offsetB].getz() - poly.vertex[offsetA].getz())*scalar);
			front_split.add(isect);
			back_split.add(isect);
			if(dA>=plane.dist_origin)
				back_split.add(poly.vertex[offsetB]);
			else
				front_split.add(poly.vertex[offsetB]);
		}
	}
	front_split.solve();
	back_split.solve();
}

/*
===========
convex
	determines if a list is elibable to become a leaf
===========
*/
bool convex(Poly_List &list) {
	Poly_List::iterator po_Iter, pi_Iter;
	Poly test_poly;
	bool front, behind;
	int status;
	// list is convex if only 1 poly or every other poly is one the same(relatively) side of a given poly
	// FIXED: Bug in 'exaflops' code   didnt reset bool front and behind after each loop iteration
	if(list.size()==1) return true;
	for(po_Iter=list.begin(); po_Iter!=list.end(); ++po_Iter) {
		front=behind=false;  // BUGFIX
		test_poly = (*po_Iter);
		for (pi_Iter=list.begin(); pi_Iter!=list.end(); ++pi_Iter) {
			if ((*pi_Iter)==test_poly)	continue;
			status = test_poly.plane.classify(*pi_Iter);
			switch(status) {
				case	FRONT:
					if(behind)
						return false;
					else
						front=true;
					break;
				case	BACK:
					if(front)
						return false;
					else
						behind=true;
					break;
				case	CO_PLANAR: break;
				case	SPANNING:	return false;
			}
		}
	}
	return true;
}

/*
===========
partition
	This is what you see very often  
	partition input list of polys against give plane
	creating front and back lists
===========
*/
void partition(Poly_List &list, const Plane &plane, Poly_List &front, Poly_List &back) {
	Poly_List::iterator p_Iter;
	Poly front_split, back_split;
	for(p_Iter=list.begin(); p_Iter!=list.end(); ++p_Iter) {
		switch (plane.classify(*p_Iter)) {
			case CO_PLANAR:  // push down either side   should intelligently choose
			case BACK:
				back.push_back(*p_Iter);
				break;
			case FRONT:
				front.push_back(*p_Iter);
				break;
			case SPANNING:
				Poly front_split, back_split;
				split(*p_Iter, plane, front_split, back_split);
				back.push_back(back_split);
				front.push_back(front_split);
				break;
		}
	}
}
#endif