#ifndef _OBJ_IO_H
#define _OBJ_IO_H

// *********************************************************
int obj_count;
bool parse_object(ifstream &, ofstream &, Poly_List &);
bool parse_sphere(ifstream &, ofstream &, Poly_List &);
bool parse_cylinder(ifstream &, ofstream &, Poly_List &);
bool parse_cone(ifstream &, ofstream &, Poly_List &);

void parse_binary_object(ifstream &, Poly_List &);
void parse_binary_sphere(ifstream &, Poly_List &);
void parse_binary_cylinder(ifstream &, Poly_List &);
void parse_binary_cone(ifstream &, Poly_List &);
// *********************************************************

// *********************************************************
bool parse_object(ifstream &vin, ofstream &bout, Poly_List &polygons) {
	Poly curr_polygon;
	Vect3D vertex[MAX_VERTEXES];

	int count, v_count, face_count, pos, list_counter, curr_vertex_num;
	int face_list[32][MAX_VERTEXES], num_in_list[32];
	char buf[128], type;
	bool done=false, dup_error=false;

	count=v_count=face_count=pos=list_counter=curr_vertex_num=0;	

	dup_error=false;

	skip_white(vin, buf);
	if(buf[0]=='v') {
		v_count=atoi(buf+1);
		if(v_count>MAX_VERTEXES) {
			error(obj_count, 0);
			return false;
		}

		for(int i=0; i<v_count; i++)
			vin >> vertex[i];
	}

	skip_white(vin, buf);
	if(buf[0]=='f') {
		face_count=atoi(buf+1);
		for(int i=0; i<face_count; i++) {
			skip_white(vin, buf);
			list_counter=count_commas(buf) + 1;
			if(list_counter==1) {
				error(obj_count, 1);
				done=true;
				break;
			}
			pos=0;
			curr_polygon.clear();
			num_in_list[i]=list_counter;
			for(int j=0; j<list_counter; j++) {
				curr_vertex_num=atoi(buf+pos);
				if(curr_vertex_num>=v_count) {
					error(obj_count, 2);
					done=true;
					break;
				}
				face_list[i][j]=curr_vertex_num;
				curr_polygon.add(vertex[curr_vertex_num]);
				pos=next_comma(buf,pos)+1;
			}
			if(done) break;
			curr_polygon.solve();
			if(!duplicate(curr_polygon, polygons)) {
				polygons.push_back(curr_polygon);
				count++;
			}
			else
				dup_error=true;
		}
		if(done) return false;
	}

	skip_white(vin, buf);
	buf[5]=0;
	if(strcmp(buf, "[end]")!=0) {
		error(obj_count, 3);
		return false;
	}

	if(dup_error)
		error(obj_count, 4);
	
	type=OBJECT;
	bout.write((unsigned char*)&type, 1);
	bout.write((unsigned char*)&v_count,4);
	bout.write((unsigned char*)&face_count,4);
	bout.write((unsigned char*)vertex, v_count*sizeof(Vect3D));
	for(int i=0; i<face_count; i++) {
		bout.write((unsigned char*)&num_in_list[i], sizeof(int));
		bout.write((unsigned char*)&face_list[i][0], num_in_list[i]*sizeof(int));
	}

	return true;
}
bool parse_sphere(ifstream &vin, ofstream &bout, Poly_List &polygons) {
	Vect3D center;
	int properties[4];
	char buf[128], type;

	skip_white(vin, buf);
	vin >> center;
	vin >> properties[0] >> properties[1] >> properties[2] >> properties[3];
	skip_white(vin, buf);
	buf[5]=0;
	if(strcmp(buf, "[end]")!=0) {
		error(obj_count, 3);
		return false;
	}
	if(properties[2]>MAX_VERTEXES) {
		error(obj_count, 0);
		return false;
	}
	if(properties[2]<3 || properties[3]<3) {
		error(obj_count, 5);
		return false;
	}

	type=SPHERE;
	bout.write((unsigned char*)&type, 1);
	bout.write((unsigned char*)&center, sizeof(Vect3D));
	bout.write((unsigned char*)properties, 4*sizeof(int));
	tessilate_sphere(center, properties[0], properties[1], properties[2], properties[3], polygons);
	return true;	
}
bool parse_cylinder(ifstream &vin, ofstream &bout, Poly_List &polygons) {
	Vect3D cent_orient[2];
	int properties[3];
	char buf[128], type;

	skip_white(vin, buf);
	vin >> cent_orient[0] >> cent_orient[1];
	vin >> properties[0] >> properties[1] >> properties[2];
	skip_white(vin, buf);
	buf[5]=0;
	if(strcmp(buf, "[end]")!=0) {
		error(obj_count, 3);
		return false;
	}
	if(properties[2]>MAX_VERTEXES) {
		error(obj_count, 0);
		return false;
	}
	if(properties[2]<3) {
		error(obj_count, 5);
		return false;
	}

	type=CYLINDER;
	bout.write((unsigned char*)&type, 1);
	bout.write((unsigned char*)cent_orient, 2*sizeof(Vect3D));
	bout.write((unsigned char*)properties, 3*sizeof(int));
	tessilate_cone(cent_orient[0], cent_orient[1], properties[0], properties[0], properties[1], properties[2], polygons);
	return true;	
}
bool parse_cone(ifstream &vin, ofstream &bout, Poly_List &polygons) {
	Vect3D cent_orient[2];
	int properties[4];
	char buf[128], type;

	skip_white(vin, buf);
	vin >> cent_orient[0] >> cent_orient[1];
	vin >> properties[0] >> properties[1] >> properties[2] >> properties[3];
	skip_white(vin, buf);
	buf[5]=0;
	if(strcmp(buf, "[end]")!=0) {
		error(obj_count, 3);
		return false;
	}
	if(properties[3]>MAX_VERTEXES) {
		error(obj_count, 0);
		return false;
	}
	if(properties[3]<3) {
		error(obj_count, 5);
		return false;
	}

	type=CONE;
	bout.write((unsigned char*)&type, 1);
	bout.write((unsigned char*)cent_orient, 2*sizeof(Vect3D));
	bout.write((unsigned char*)properties, 4*sizeof(int));
	tessilate_cone(cent_orient[0], cent_orient[1], properties[0], properties[1], properties[2], properties[3], polygons);
	return true;	
}
// *********************************************************

// *********************************************************
void parse_binary_object(ifstream &bin, Poly_List &polygons) {
	Poly curr_poly;
	Vect3D vertex[MAX_VERTEXES];
	int properties[2];
	int num_in_list[MAX_VERTEXES], face_list[32][MAX_VERTEXES];

	bin.read((unsigned char*)properties, 2*sizeof(int));
	bin.read((unsigned char*)vertex, properties[0]*sizeof(Vect3D));
	for(int i=0; i<properties[1]; i++) {
		bin.read((unsigned char*)&num_in_list[i], sizeof(int));
		bin.read((unsigned char*)&face_list[i][0], num_in_list[i]*sizeof(int));
	}

	for(int i=0; i<properties[1]; i++) {
		curr_poly.clear();
		for(int j=0; j<num_in_list[i]; j++)
			curr_poly.add(vertex[face_list[i][j]]);
		curr_poly.solve();
		polygons.push_back(curr_poly);
	}
}
void parse_binary_sphere(ifstream &bin, Poly_List &polygons) {
	Vect3D center;
	int properties[4];

	bin.read((unsigned char*)&center,sizeof(Vect3D));
	bin.read((unsigned char*)properties, 4*sizeof(int));
	tessilate_sphere(center, properties[0], properties[1],  properties[2], properties[3], polygons);
}
void parse_binary_cylinder(ifstream &bin, Poly_List &polygons) {
	Vect3D cent_orient[2];
	int properties[3];

	bin.read((unsigned char*)cent_orient,2*sizeof(Vect3D));
	bin.read((unsigned char*)properties, 3*sizeof(int));
	tessilate_cone(cent_orient[0], cent_orient[1], properties[0], properties[0], properties[1], properties[2], polygons);
}
void parse_binary_cone(ifstream &bin, Poly_List &polygons) {
	Vect3D cent_orient[2];
	int properties[4];

	bin.read((unsigned char*)cent_orient,2*sizeof(Vect3D));
	bin.read((unsigned char*)properties, 4*sizeof(int));
	tessilate_cone(cent_orient[0], cent_orient[1], properties[0], properties[1], properties[2], properties[3], polygons);
}
#endif